#!/usr/bin/perl

package MailScanner::CustomConfig;

use strict;
use Fcntl qw(:DEFAULT :flock);
use NDBM_File;



my $ECSthresholds = "/opt/MailScanner/var/spamthreshold.db";
my $lock_file = $ECSthresholds . ".lock";

my %ECSthresholds;

sub InitECSthreshold {
  my($result, %diskstore, $tries);

  MailScanner::Log::InfoLog("Trying to open threshold db %s", $ECSthresholds);

  # Try to open the database 10 times
  for $tries (1..10) {
    open(LOCK, "+> $lock_file") or do {
      MailScanner::Log::WarnLog("Failed to open threshold db lock on attempt %d",
                                $tries);
      next;
    };
    flock(LOCK, LOCK_SH | LOCK_NB) or do {
      MailScanner::Log::WarnLog("Failed to get threshold db lock on attempt %d",
                                $tries);
      next;
    };
    tie(%diskstore, "NDBM_File", $ECSthresholds, O_RDONLY, 0666) or do {
      MailScanner::Log::WarnLog("Failed to open threshold db on attempt %d",
                                $tries);
      next;
    };
    #print STDERR "Lookup of \@ produced \"" . $diskstore{'@'} . "\"\n";
    unless (defined($diskstore{'@'}) && $diskstore{'@'} eq '@') {
        MailScanner::Log::WarnLog("Opened db but it was missing '\@' record");
	next;
    }

    %ECSthresholds = %diskstore;
    untie %diskstore;
    flock(LOCK, LOCK_UN);
    close(LOCK);
    MailScanner::Log::InfoLog("Successfully read threshold db");
    return 0;

  } continue {
    sleep 10;
  }

  MailScanner::Log::WarnLog("Gave up trying to access db");
}

sub ECSthreshold {
  my($message) = @_;
  my($to, $greatest, $lowercase);

  return 6 unless $message;
  return 6 unless @{$message->{to}};

  # Find the greatest value among all the recipients' thresholds
  foreach $to (@{$message->{to}}) {
    $lowercase = lc($to);
    $greatest = $ECSthresholds{$lowercase}
      if $ECSthresholds{$lowercase}>$greatest;
  }

  $greatest = 6 unless $greatest;
  MailScanner::Log::InfoLog("ECSthresholds: Returning value %d for %s",
                            $greatest, join(',',@{$message->{to}}));
  return $greatest;
}

sub EndECSthreshold {
  1;
}

sub InitECSspamaction {
  1;
}

sub ECSspamaction {
  my($message) = @_;
  my($to, $action, $lowercase, $changed, %recips);

  return 'deliver' unless $message;
  return 'deliver' unless @{$message->{to}};

  $action = 'deliver';

  # Build the hash of recipients and say we have not changed anything
  $changed = 0;
  foreach $to (@{$message->{to}}) {
    $lowercase = lc($to); # To avoid occasional bug in Perl
    $recips{$lowercase} = 1;
  }

  foreach $to (@{$message->{to}}) {
    $lowercase = lc($to);
    if (exists $ECSthresholds{$lowercase}) {
      delete $recips{$lowercase};
      $changed = 1;
    }
  }

  # If nothing has changed, just deliver the message
  unless ($changed) {
    MailScanner::Log::InfoLog("ECSthresholds: Returning action deliver %s",
                            join(',',@{$message->{touser}}));
    return 'deliver';
  }

  my(@newto, @newtodomain, @newtouser);
  my($user, $domain);

  # Have we deleted all the recipients?
  @newto = keys %recips;
  unless (@newto) {
    MailScanner::Log::InfoLog("ECSthresholds: Returning action delete " .
                 "for all recipients: %s", join(',',@{$message->{touser}}));
    return 'delete';
  }

  # We know we have deleted some of the recipients but not all.
  # Delete the recipients from the message, add the new list back in,
  # then setup all the message data structures for the new recipient list.
  $global::MS->{mta}->DeleteRecipients($message);
  $global::MS->{mta}->AddRecipients($message, @newto);

  # Work out the user @ domain components
  foreach $to (@newto) {
    ($user, $domain) = MailScanner::Message::address2userdomain($to);
    push @newtouser, $user;
    push @newtodomain, $domain;
  }

  #Now push the new recipients details back into the message structures
  @{$message->{to}} = @newto;
  @{$message->{touser}} = @newtouser;
  @{$message->{todomain}} = @newtodomain;

  MailScanner::Log::InfoLog("ECSthresholds: Reducing recipient list to %s",
                            join(',',@{$message->{touser}}));
  return 'deliver';
}

sub EndECSspamaction {
  1;
}

1;

